@@ -3,7 +3,8 @@
 	if(any(is.na(internal_ids))) stop(paste("Indicated ID(s) not among original ID labels:", paste(orig.ids[is.na(internal_ids)], collapse=",")))
 	internal_ids
 }
-.getSex = function(x, ids) {as.vector(x$pedigree[match(ids, x$orig.ids), 'SEX'])}
+.getSex = function(x, orig.ids) as.vector(x$pedigree[.internalID(x, orig.ids), 'SEX'])
+
 .comb2 = function(n) {
 		if (n<2) return(matrix(nrow=0, ncol=2))
 		v1 = rep.int(seq_len(n-1), (n-1):1)
@@ -28,7 +29,6 @@ spouses <- function(x, id, original.id=TRUE) { #returns a vector containing all
 	if (original.id) return(x$orig.ids[spou]) else return(spou)
 }	
 
-
 descendants=function(x, id, original.id=TRUE) { 
 	internal_id = ifelse(original.id, .internalID(x, id), id)
 	nextgen <- desc <- offspring(x, internal_id, original.id=FALSE)
@@ -41,7 +41,6 @@ descendants=function(x, id, original.id=TRUE) {
 	if (original.id) return(x$orig.ids[desc]) else return(desc)
 }
 
-
 #pedDistMatrix(x) computes a symmetrix integer matrix (d_ij), where d_ij is the pedigree distance (i.e., the length of the shortest pedigree path) between i and j. For example, the following relations have the indicated pedigree distances: parent/offspring	1, siblings	2, parents	2, uncle/niece	3, first cousins 4.
 .pedDistMatrix <- function(x) { 
 	d = matrix(0, ncol=x$nInd, nrow=x$nInd)
@@ -132,35 +131,102 @@ pedigreeLoops = function(x) {
 	sibs
 }
 
-uninformative = function(x, return.reduced.ped=FALSE) {
-	p = relabel(x$pedigree, x$orig.ids)
+.trim = function(x, return.ids=TRUE) {
+	store = paramlink:::.as.annotated.matrix(x); 	store_attrs = attributes(store)
+	
 	remove_IDs = function(p, ids) p[!p[,'ID'] %in% ids, ]
 	
+	y = linkdat(relabel(x$pedigree, x$orig.ids), verbose=F)
+	y$available = x$available
 	uninformative = numeric(0)
 	while(TRUE) {
-		ID = p[, 'ID']
-		leaves = setdiff(ID, p[, c('FID','MID')])
-		nonaff = p[ p[,'AFF']<2 , 'ID' ]; healthy = p[ p[,'AFF']==1 , 'ID' ]; unknown = p[ p[,'AFF']==0 , 'ID' ]
-		uninf = intersect(leaves, unknown) #these are always uninformative
-
-		healthy_leaves = intersect(leaves, healthy)
-		nonaff_parents = (p[match(healthy_leaves, ID), 'FID'] %in% nonaff) & (p[match(healthy_leaves, ID), 'MID'] %in% nonaff)
-		maybe = healthy_leaves[nonaff_parents] #these are healthy leaves with healthy or unknown parents. Will be marked as uninf iff all sibs are healthy
-		if(length(maybe) > 0) {
-			healthy_sibs = sapply(maybe, function(id) {intid = match(id, ID); sibs = p[ (p[, 'FID'] == p[intid, 'FID']) & (p[, 'MID'] == p[intid, 'MID']), 'ID']; all(sibs %in% healthy)})
-			uninf = c(uninf, maybe[healthy_sibs])
+		p = .as.annotated.matrix(y); attrs = attributes(p)
+		leaves = setdiff(p[, 'ID'], p[, c('FID','MID')])
+		leaves_intern = .internalID(y, leaves)
+		unknown = p[ p[,'AFF']==0 , 'ID' ]
+		uninf = intersect(leaves, unknown) #leaves with unknown affection status are always uninformative, independent of availability.
+		for(s in y$subnucs) { #NB: inside this loop all IDs are internal
+			offs = s[-(1:3)]
+			offs_unavail = !y$orig.ids[offs] %in% y$available
+			offs_leaves = offs %in% leaves_intern
+			uninf = c(uninf, y$orig.ids[offs[offs_unavail & offs_leaves]])
+			if(all(offs_unavail & offs_leaves) && length(fp <- intersect(s[2:3], y$founders)) == 1)
+				uninf = c(uninf, y$orig.ids[fp]) #if all offspring are trimmed, a single founder parent must follow.
 		}
-		if(length(uninf)==0) break
 		
-		p = remove_IDs(p, uninf)
-		fou = p[ p[, 'FID']==0 & p[, 'MID']==0, 'ID']
-		leftover_spouses = setdiff(fou, p[, c('FID','MID')])
-		p = remove_IDs(p, leftover_spouses)
-		uninformative = c(uninformative, uninf, leftover_spouses)
+		if(length(uninf)==0) break
+		uninformative = c(uninformative, uninf)
+		y = .restore.linkdat(remove_IDs(p, uninf), attrs=attrs)
 	}
-	if(return.reduced.ped) {
-		cat("Removing individuals:", paste(sort(uninformative), collapse=", "), "\n") 
-		return(linkdat(p, model=x$model, verbose=FALSE))
-	}
-	else return(sort(uninformative))
+	uninformative = sort(unique(uninformative))
+	if(return.ids) return(uninformative)
+	
+	cat("Removing individuals:", paste(uninformative, collapse=", "), "\n") 
+	return(.restore.linkdat(remove_IDs(store, uninformative), attrs=store_attrs))
 }
+
+
+breakLoops = function(x, loop_breakers) {
+	stopifnot(is.numeric(loop_breakers), length(loop_breakers)>0)
+	if(any(x$orig.ids[loop_breakers] %in% x$founders)) stop("Pedigree founders cannot be loop breakers.")
+
+	pedm = .as.annotated.matrix(x)#data.frame(x, famid=T, missing=0)
+	attrs = attributes(pedm) #all attributes except 'dim'
+	dup_pairs = x$loop_breakers #normally = NULL at this stage 
+	for(id in loop_breakers) {
+		dup_id = max(pedm[, 'ID']) + 1
+		dup_pairs = rbind(dup_pairs, c(id, dup_id))
+		intern = match(id, pedm[,'ID'])  #don't use .internalID here, since pedm changes all the time
+		sex_col = pedm[intern, 'SEX'] + 2 # FID column if 'intern' is male; MID if female
+		
+		pedm = pedm[c(1:intern, intern, (intern+1):nrow(pedm)), ]
+		pedm[intern+1, c('ID', 'FID','MID')] = c(dup_id, 0, 0)
+		pedm[pedm[, sex_col] == id, sex_col] = dup_id
+	}
+	newx = .restore.linkdat(pedm, attrs=attrs) 
+	newx$loop_breakers = dup_pairs
+	newx
+}
+
+tieLoops = function(x) {
+	dups = x$loop_breakers
+	if(is.null(dups) || nrow(dups)==0) {cat("No loops to tie\n"); return(x)}
+	if(!all(dups %in% x$orig.ids)) stop("Something's wrong: Duplicated individuals no longer in pedigree.")
+	pedm = .as.annotated.matrix(x)
+	attrs = attributes(pedm)
+
+	origs = dups[,1]; copies = dups[,2]
+	pedm = pedm[-match(copies, pedm[,'ID']), ]
+	for(i in 1:length(origs)) {
+		orig = origs[i]; copy = copies[i]
+		sex = pedm[pedm[,'ID']==orig, 'SEX']
+		pedm[pedm[, sex+2] == copy, sex+2] = orig 
+	}
+	.restore.linkdat(pedm, attrs=attrs)
+}
+
+all.equal.linkdat = function(target, current, ...) {
+	if(class(target)!='linkdat' || class(current)!='linkdat') return(F)
+	if(!setequal(target$orig.ids, current$orig.ids)) {
+		cat("ID labels are not equal\n"); return(FALSE)
+	}
+	names.t = names(target)
+	names.c = names(current)
+	if(!setequal(names.t, names.c)) {
+		first_miss = setdiff(names.c, names.t)
+		sec_miss = setdiff(names.t, names.c)
+		if(length(first_miss)>0)
+			cat("Missing slots in first object:\n", paste(first_miss, sep=","), "\n")
+		if(length(sec_miss)>0)
+			cat("Missing slots in second object:\n", paste(sec_miss, sep=","), "\n")
+		return(FALSE)
+	}
+	target_m = .as.annotated.matrix(target)
+	target = .restore.linkdat(target_m[match(current$orig.ids, target$orig.ids), ], attrs=attributes(target_m))
+	test = sapply(names.t, function(name) isTRUE(all.equal(target[[name]], current[[name]])))
+	if(!all(test)) {
+		cat("Difference detected in the following slots:\n", paste(names.t[!test], sep=","), "\n")
+		return(FALSE)
+	}
+	return(TRUE)
+}
\ No newline at end of file
